@@ -1,4 +1,5 @@
 # frozen_string_literal: true
+
 module Molinillo
   class DependencyGraph
     # An action that modifies a {DependencyGraph} that is reversible.
@@ -7,7 +8,7 @@ module Molinillo
       # rubocop:disable Lint/UnusedMethodArgument
 
       # @return [Symbol] The name of the action.
-      def self.name
+      def self.action_name
         raise 'Abstract'
       end
 
